import React, { useEffect, useState } from "react";
import "./App.css";
import GrammarList from "./components/GrammarList";
import GrammarAdder from "./components/GrammarAdder";
import { Searches } from "./components/Searches";
import { MdRefresh } from "react-icons/md";

function App() {
  const [grammars, setGramars] = useState([]);
  const [add, setAdd] = useState(false);
  const [filteredGrammars, setFilteredGrammars] = useState(null);

  const getGrammars = async () => {
    await fetch("http://localhost:8080/api/grammars/all")
      .catch((error) => {
        alert(error);
      })
      .then((resp) => resp.json())
      .then((grammarsResp) => setGramars(grammarsResp));
  };
  useEffect(() => {
    getGrammars();
  }, []);

  function Grammar(id, uri, rules, rootRule) {
    this.id = id;
    this.uri = uri;
    this.rules = rules;
    this.rootRule = rootRule;
  }

  const deleteGramar = async (id) => {
    const response = await fetch("http://localhost:8080/api/grammars/" + id, {
      method: "DELETE",
    });
    setGramars(grammars.filter((grammar) => grammar.id != id));
  };

  const showForm = () => {
    setAdd(!add);
  };

  const duplicateGr = async (id) => {
    const response = await fetch(
      "http://localhost:8080/api/grammars/" + id + "/duplicate/",
      {
        method: "POST",
      }
    );
    const grammar = await response.json();
    setGramars([...grammars, grammar]);
  };

  const save = async (uri, rules, rootRule) => {
    const response = await fetch("http://localhost:8080/api/grammars/", {
      method: "POST",
      headers: {
        "Content-type": "application/json",
      },
      body: JSON.stringify(new Grammar(null, uri, rules, rootRule)),
    });
    if (response.status != 201) {
      setAdd(false);
      return;
    }
    const grammar = await response.json();
    if (response.status != 201) {
      alert(grammar.message);
      return;
    }
    setGramars([...grammars, grammar]);

    setAdd(false);
  };

  const editGramar = async (Grammar) => {
    const response = await fetch(
      "http://localhost:8080/api/grammars/" + Grammar.id + "/uri/",
      {
        method: "PUT",
        headers: {
          "Content-type": "application/json",
        },
        body: JSON.stringify(Grammar),
      }
    );
    const resp_grammar = await response.json();

    setGramars(
      grammars.map((grammar) =>
        grammar.id == Grammar.id ? resp_grammar : grammar
      )
    );
  };

  const getGrammar = async (grammarId) => {
    const response = await fetch(
      "http://localhost:8080/api/grammars/" + grammarId
    );

    const resp_grammar = await response.json();
    setGramars(
      grammars.map((grammar) =>
        grammar.id == grammarId ? resp_grammar : grammar
      )
    );
  };

  const addRule = async (rule, grammarId) => {
    const response = await fetch(
      "http://localhost:8080/api/rules/" + grammarId,
      {
        method: "POST",
        headers: {
          "Content-type": "application/json",
        },
        body: JSON.stringify(rule),
      }
    );
    const newRule = await response.json();
    if (!(response.status == 201 || response.status == 200)) {
      alert(newRule.message);
      return;
    }
    if (newRule.root) {
      setGramars(
        grammars.map((grammar) =>
          grammar.id == grammarId ? (grammar.rootRule = newRule) : grammar
        )
      );
    }
    if (response.status == 201) {
      setGramars(
        grammars.map((grammar) =>
          grammar.id == grammarId
            ? { ...grammar, rules: [...grammar.rules, newRule] }
            : grammar
        )
      );
      return;
    }
    if (response.status == 200) {
      setGramars(
        grammars.map((grammar) =>
          grammar.id == grammarId
            ? {
                ...grammar,
                rules: grammar.rules.map((rule) =>
                  newRule.id == rule.id ? newRule : rule
                ),
              }
            : grammar
        )
      );
      return;
    }
  };
  const deleteRule = async (ruleId, root, grammarId) => {
    await fetch("http://localhost:8080/api/rules/" + ruleId, {
      method: "DELETE",
    });

    setGramars(
      grammars.map((grammar) =>
        grammar.id == grammarId
          ? {
              ...grammar,
              rules: grammar.rules.filter((rule) => rule.id != ruleId),
              rootRule: root ? null : grammar.rootRule,
            }
          : grammar
      )
    );
  };

  const deleteFilesFromGrammar = async (id) => {
    var res = window.confirm(
      "Confirm clearing of files from grammar id: " + id
    );
    if (res) {
      console.log("clear-confirmed");
    } else {
      return;
    }
    const response = await fetch(
      "http://localhost:8080/api/files/grammar/" + id,
      {
        method: "DELETE",
      }
    );
    setGramars(
      grammars.map((grammar) =>
        grammar.id == id ? { ...grammar, files: [] } : grammar
      )
    );
  };

  const clear = async () => {
    var res = window.confirm("Confirm clearing of grammars");
    if (res) {
      console.log("clear-confirmed");
    } else {
      return;
    }
    const response = await fetch("http://localhost:8080/api/grammars/all", {
      method: "DELETE",
    });
    setGramars([]);
  };

  const changeRule = async (ruleId, rule, grammarId) => {
    const response = await fetch("http://localhost:8080/api/rules/" + ruleId, {
      method: "PUT",
      headers: {
        "Content-type": "application/json",
      },
      body: JSON.stringify(rule),
    });
    const newRule = await response.json();
    /* setGramars(
      grammars.map((grammar) =>
        grammar.id == grammarId
          ? {
              ...grammar,
              rules: grammar.rules.map((oldRule) =>
                oldRule.id == ruleId ? newRule : oldRule
              ),
            }
          : grammar
      )
    );*/
    getGrammar(grammarId);
  };

  const addFromFile = async (file, xml, save) => {
    var header = {
      "Content-type": xml ? "aplication/xml" : "text/plain",
    };
    var endpoint =
      "http://localhost:8080/api/files/" +
      (xml ? "xml" : "jsgf") +
      "?save=" +
      save;
    const response = await fetch(endpoint, {
      method: "POST",
      headers: header,
      body: file,
    });

    const grammar = await response.json();
    if (response.status != 201) {
      alert(grammar.message);
      return;
    }
    setGramars([...grammars, grammar]);
  };

  const fetchFileHolders = async (id) => {
    var endpoint = "http://localhost:8080/api/files/grammar/" + id;
    const response = await fetch(endpoint);
    const files = await response.json();

    setGramars(
      grammars.map((grammar) => (id == grammar.id ? { ...grammar } : grammar))
    );
  };
  const getFile = async (id, grammarId) => {
    var endpoint = "http://localhost:8080/api/files/" + id;
    const response = await fetch(endpoint);
    const text = await response.text();
    setGramars(
      grammars.map((grammar) =>
        grammarId == grammar.id
          ? {
              ...grammar,
              files: grammar.files.map((file) =>
                file.id == id ? { ...file, content: text } : file
              ),
            }
          : grammar
      )
    );
  };

  const generateFile = async (xml, grammarId) => {
    var endpoint =
      "http://localhost:8080/api/files/" +
      grammarId +
      "/new/" +
      (xml ? "xml" : "jsgf");
    const response = await fetch(endpoint);
    const text = await response.text();
    if (xml) {
      setGramars(
        grammars.map((grammar) =>
          grammarId == grammar.id ? { ...grammar, xmlGen: text } : grammar
        )
      );
      return;
    }
    setGramars(
      grammars.map((grammar) =>
        grammarId == grammar.id ? { ...grammar, jsgfGen: text } : grammar
      )
    );
  };
  const fileToGrammar = async (file, xml, grammarId, save, diff) => {
    var header = {
      "Content-type": xml ? "aplication/xml" : "text/plain",
    };
    var endpoint =
      "http://localhost:8080/api/files/" +
      grammarId +
      (xml ? "xml" : "jsgf") +
      "?save=" +
      save +
      "&newGrammar=" +
      diff;
    const response = await fetch(endpoint, {
      method: "POST",
      headers: header,
      body: file,
    });

    if (response.status == 204) {
      alert("different grammar");
      return;
    }
    const grammarNew = await response.json();
    if (response.status == 201) {
      setGramars([...grammars, grammarNew]);
      return;
    }

    if (response.status == 200) {
      setGramars(
        grammars.map((grammar) =>
          grammar.id == grammarId
            ? { ...grammar, files: grammarNew.files }
            : grammar
        )
      );
      return;
    }
    alert(response.message);
  };
  //todo better files get
  const deleteFile = async (fileId, grammarId) => {
    const response = await fetch("http://localhost:8080/api/files/" + fileId, {
      method: "DELETE",
    });
    setGramars(
      grammars.map((grammar) =>
        grammar.id == grammarId
          ? {
              ...grammar,
              files: grammar.files.filter((file) => file.id != fileId),
            }
          : grammar
      )
    );
  };

  const changeRoot = async (id, grammarId, oldRule) => {
    var newRule = { root: !oldRule.root };
    const response = await fetch("http://localhost:8080/api/rules/" + id, {
      method: "PUT",
      headers: {
        "Content-type": "application/json",
      },
      body: JSON.stringify(newRule),
    });
    var root = oldRule.root;
    setGramars(
      grammars.map((grammar) =>
        grammar.id == grammarId
          ? {
              ...grammar,
              rootRule: root ? null : oldRule,
              rules: grammar.rules.map((rule) =>
                rule.id == id ? { ...rule, root: !root } : rule
              ),
            }
          : grammar
      )
    );
  };

  return (
    <div className="App">
      <h1>
        Grammar Manager <MdRefresh onClick={getGrammars} />
      </h1>

      <Searches setFilteredGrammars={setFilteredGrammars} grammars={grammars} />

      <button onClick={clear}>CLEAR</button>
      {grammars.length != 0 &&
      (filteredGrammars == null || filteredGrammars.length != 0) ? (
        <GrammarList
          grammars={filteredGrammars == null ? grammars : filteredGrammars}
          deleteGr={deleteGramar}
          editGr={editGramar}
          addRule={addRule}
          duplicateGr={duplicateGr}
          deleteRule={deleteRule}
          changeRule={changeRule}
          generateFile={generateFile}
          changeRoot={changeRoot}
          getFile={getFile}
          fileToGrammar={fileToGrammar}
          deleteFile={deleteFile}
          deleteFilesFromGrammar={deleteFilesFromGrammar}
        />
      ) : (
        "No grammars to show"
      )}
      <button onClick={showForm}>{add ? "CANCEL" : "ADD GRAMMAR"}</button>
      {add && <GrammarAdder save={save} fileAdd={addFromFile} />}
      <footer>
        <h1>PADDING</h1>
      </footer>
    </div>
  );
}

export default App;
